"use strict";
// A Typescript interpretation of the JSON spec from
// http://www.ecma-international.org/publications/files/ECMA-ST/ECMA-404.pdf
Object.defineProperty(exports, "__esModule", { value: true });
// ENFORCED indicates that the field is enforced by the type system, and it should be
// impossible for any type assignable to Validated<schema> to fail JSON validation because of constraints
// that this field introduces.
// For example, the `required` field on objects is ENFORCED because a type assignable to Validated<T>
// must have all fields marked `required`.
// PARTIALLY ENFORCED indicates that the field is partially enforced by the type system, but it may be possible
// to assign a type to Validated<schema> that does not validate.
// For example, arrays with the additionalItems parameter are PARTIALLY ENFORCED becuase (currently) every element
// in the validated type can be assigned to the additionalItems type, when only items after items.length should
// be validated against this schema.
// NOT ENFORCED indicates that the field is not enforced by the type system. This is either because it's impossible
// to do so efficiently given Typescript, or because I haven't figured out how yet. If the latter, hopefully I've
// included a comment.
// For example, the `pattern` constraint in a string type is NOT ENFORCED because there's no reasonable way to
// express a type that means "a string that matches this regex".
// NO ENFORCEMENT NEEDED means that this field does not add any constraints to a JSON schema so is essentially a comment.
// Symbol needed to compile a program that passes typechecking but still fails schema validation.
exports.InternalTypeSymbol = Symbol("InternalType");
/**
 * Create a schema that validates strings.
 *
 * If the `enum` property is specified, it will by typed as a union of these enum values.
 * @param options
 */
const stringConstructor = (options = {}) => {
    return Object.assign(Object.assign({ type: "string" }, options), { [exports.InternalTypeSymbol]: {} });
};
/**
 * Create a schema that validates numbers.
 *
 * @param options
 */
const numberConstructor = (options = {}) => {
    return Object.assign(Object.assign({ type: "number" }, options), { [exports.InternalTypeSymbol]: {} });
};
/**
 * Create a schema that validates integers.
 *
 * @param options
 */
const integerConstructor = (options = {}) => {
    return Object.assign(Object.assign({ type: "integer" }, options), { [exports.InternalTypeSymbol]: {} });
};
/**
 * Create a schema that validates objects.
 *
 * The `properties` field defines the keys and schemas that these properties must conform to.
 * `requiredProperties` marks zero or more fields required.
 * `additionalProperties` marks the types of the fields not specified in properties.
 * @param options
 */
const objectConstructor = (options) => {
    return Object.assign(Object.assign({ type: "object" }, options), { [exports.InternalTypeSymbol]: {} });
};
/**
 * Create a schema that validates arrays.
 *
 * The type of the array is defined by the `items` parameter, which is either
 * a single JSON schema or a list of JSON schemas. If `items` is a single schema,
 * all members of the array must conform to that schema. If it's a list of schemas,
 * each member of the array must validate to its respective schema, and all remaining
 * members must validate to the schema in AdditionalItems.
 * @param options
 */
const arrayConstructor = (options = {}) => {
    return Object.assign(Object.assign({ type: "array" }, options), { [exports.InternalTypeSymbol]: {} });
};
/**
 * Create a schema that validates boolean values.
 *
 * @param options
 */
const booleanConstructor = (options = {}) => {
    return Object.assign(Object.assign({ type: "boolean" }, options), { [exports.InternalTypeSymbol]: {} });
};
/**
 * Create a schema that validates null values.
 *
 * @param options
 */
const nullConstructor = (options = {}) => {
    return Object.assign(Object.assign({ type: "null" }, options), { [exports.InternalTypeSymbol]: {} });
};
/**
 * Create a schema that validates if any schemas in its anyOf parameter validate.
 *
 * @param options
 */
const anyOfConstructor = (options) => {
    return Object.assign(Object.assign({}, options), { [exports.InternalTypeSymbol]: {} });
};
/**
 * Create a schema that validates if all schemas in its allOf parameter validate.
 *
 * @param options
 */
const allOfConstructor = (options) => {
    return Object.assign(Object.assign({}, options), { [exports.InternalTypeSymbol]: {} });
};
exports.Schema = {
    String: stringConstructor,
    Number: numberConstructor,
    Integer: integerConstructor,
    Object: objectConstructor,
    Array: arrayConstructor,
    Boolean: booleanConstructor,
    Null: nullConstructor,
    AnyOf: anyOfConstructor,
    AllOf: allOfConstructor
};
